@@ -261,4 +261,58 @@
         (should (= exit-code 0))))))
 
 
+(ert-deftest eldev-vc-repositories-setup-form ()
+  (eldev--test-with-temp-copy "dependency-b" 'Git
+    (let* ((dependency-b-dir    eldev--test-project)
+           (eldev--test-project "vc-dep-project-c"))
+      (eldev--test-delete-cache)
+      ;; It's not enough to provide `dependency-b', because it doesn't explain where to find its own
+      ;; dependencies (it needs `dependency-a'), so Eldev cannot build a package out of it.
+      (eldev--test-run nil ("--setup" `(eldev-use-vc-repository 'dependency-b :git ,dependency-b-dir)
+                            ;; This `eldev-use-package-archive' call applies to outer Eldev (for
+                            ;; `vc-dep-project-c'), so it doesn't help the inner Eldev.
+                            "--setup" `(eldev-use-package-archive `("archive-a" . ,(expand-file-name "../package-archive-a")))
+                            "prepare")
+        (should (string-match-p "dependency-a.+not available" stderr))
+        (should (= exit-code 1)))
+      (eldev--test-run nil ("--setup" `(eldev-use-vc-repository 'dependency-b :git ,dependency-b-dir
+                                                                ;; But this can be resolved by providing `:setup' to `eldev-use-vc-repository' call.
+                                                                :setup `(eldev-use-package-archive '("archive-a" . ,(expand-file-name "../package-archive-a"))))
+                            "--setup" `(eldev-use-package-archive `("archive-a" . ,(expand-file-name "../package-archive-a")))
+                            "prepare")
+        (should (= exit-code 0))))))
+
+(eldev-ert-defargtest eldev-vc-repositories-with-source-dirs (set-using-setup)
+                      (nil t)
+  (eldev--test-with-temp-copy "dependency-d" 'Git
+    (let* ((dependency-d-dir    eldev--test-project)
+           (eldev--test-project "vc-dep-project-d"))
+      ;; `dependency-d' actually has file `Eldev', where it declares that the sources can be found in
+      ;; subdirectory `lisp'.  Optionally, we also test that this information can be supplied via `:setup'
+      ;; form passed through `eldev-use-vc-repository', see issue #114.
+      (when set-using-setup
+        (let ((default-directory dependency-d-dir))
+          (delete-file "Eldev")
+          (eldev-call-process (eldev-git-executable) `("commit" "Eldev" "--message=remove file 'Eldev'"))))
+      (eldev--test-delete-cache)
+      (eldev--test-run nil ("--setup" `(eldev-use-vc-repository 'dependency-d :git ,dependency-d-dir
+                                                                ,@(when set-using-setup `(:setup `(setf eldev-project-source-dirs "lisp"))))
+                            "prepare")
+        (should (= exit-code 0))))))
+
+
+(ert-deftest eldev-vc-repositories-wrong-package ()
+  (eldev--test-with-temp-copy "dependency-e" 'Git
+    (let ((dependency-e-dir    eldev--test-project)
+          (eldev--test-project "vc-dep-project-a"))
+      (eldev--test-delete-cache)
+      ;; Make sure that such misconfiguration, i.e. when the repository doesn't provide the "promised"
+      ;; package, are caught early and result in an understandable failure.
+      (eldev--test-run nil ("--setup" `(eldev-use-vc-repository 'dependency-a :git ,dependency-e-dir)
+                            "prepare")
+        (should (string-match-p (rx-to-string `(seq ,dependency-e-dir (0+ any) "failed to provide" (0+ any) "dependency-a")) stderr))
+        (should (string-match-p (rx "repository contains package" (0+ any) "dependency-e") stderr))
+        (should (= exit-code 1))))))
+
+
 (provide 'test/vc-repositories)
